package com.nationrel.modules.rp.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.google.common.collect.Maps;
import com.nationrel.modules.rp.aop.UpdatePrediction;
import com.nationrel.modules.rp.constant.RpConstant;
import com.nationrel.modules.rp.dto.PredictionStruTreeImport;
import com.nationrel.modules.rp.entity.PredictionColumns;
import com.nationrel.modules.rp.entity.PredictionInfo;
import com.nationrel.modules.rp.entity.PredictionStruTree;
import com.nationrel.modules.rp.entity.PredictionTable;
import com.nationrel.modules.rp.mapper.PredictionColumnsMapper;
import com.nationrel.modules.rp.mapper.PredictionInfoMapper;
import com.nationrel.modules.rp.mapper.PredictionStruTreeMapper;
import com.nationrel.modules.rp.mapper.PredictionTableMapper;
import com.nationrel.modules.rp.req.PasteStructureReq;
import com.nationrel.modules.rp.req.SaveStructureReq;
import com.nationrel.modules.rp.resp.PredictionColumnsResp;
import com.nationrel.modules.rp.resp.PredictionStruTreeResp;
import com.nationrel.modules.rp.resp.QueryPredictionStruTreeResp;
import com.nationrel.modules.rp.service.IPredictionStruTreeService;
import me.zhyd.oauth.utils.UuidUtils;
import org.apache.commons.compress.utils.Lists;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.exception.NationrelBootException;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description: prediction_stru_tree
 * @Author: nationrel
 * @Date:   2024-08-26
 * @Version: V1.0
 */
@Service
public class PredictionStruTreeServiceImpl extends ServiceImpl<PredictionStruTreeMapper, PredictionStruTree> implements IPredictionStruTreeService {

    @Autowired
    private PredictionStruTreeMapper predictionStruTreeMapper;
    @Autowired
    private PredictionInfoMapper predictionInfoMapper;
    @Autowired
    private PredictionTableMapper predictionTableMapper;
    @Autowired
    private PredictionColumnsMapper predictionColumnsMapper;

    /**
     * @Description 获取左侧树
     * @param predictionId
     * @param struId
     * @return java.util.List<com.nationrel.modules.rp.resp.PredictionStruTreeResp>
     * @Author nijun
     * @Date: 2024/9/2 15:04
     */
    @Override
    public QueryPredictionStruTreeResp getLeftTree(Long predictionId, Long struId) {
        QueryPredictionStruTreeResp struTreeResp = new QueryPredictionStruTreeResp();
        int leftNo, rightNo;
        if (ObjectUtils.isEmpty(struId)) {
            PredictionStruTree predictionStruTree = this.getOne(new LambdaQueryWrapper<PredictionStruTree>()
                    .eq(PredictionStruTree::getPredictionId, predictionId)
                    .eq(PredictionStruTree::getParentStruId, RpConstant.ROOTNODE_PARENT));
            // 取得根节点
            leftNo = predictionStruTree.getLeftNo();
            rightNo = predictionStruTree.getRightNo();
        } else {
            PredictionStruTree predictionStruTree = this.getOne(new LambdaQueryWrapper<PredictionStruTree>()
                    .eq(PredictionStruTree::getPredictionId, predictionId)
                    .eq(PredictionStruTree::getId, struId));
            leftNo = predictionStruTree.getLeftNo() + 1;
            rightNo = predictionStruTree.getRightNo() - 1;
        }

        List<PredictionStruTree> predictionStruTreeList = this.list(new LambdaQueryWrapper<PredictionStruTree>()
                .eq(PredictionStruTree::getPredictionId, predictionId)
                .eq(PredictionStruTree::getIsDelete, RpConstant.NOT_DELETE)
                .between(PredictionStruTree::getLeftNo, leftNo, rightNo));
        List<PredictionTable> predictionTableList = predictionTableMapper.selectList(new LambdaQueryWrapper<PredictionTable>()
                .select(PredictionTable::getId, PredictionTable::getStruId,
                        PredictionTable::getColumnsId, PredictionTable::getValue)
                .eq(PredictionTable::getPredictionId, predictionId));
        Map<String, List<PredictionTable>> tableMap = predictionTableList.stream().collect(Collectors.groupingBy(PredictionTable::getStruId));
        List<PredictionColumns> predictionColumnList = predictionColumnsMapper.selectList(new LambdaQueryWrapper<PredictionColumns>()
                .select(PredictionColumns::getId, PredictionColumns::getColName, PredictionColumns::getColType,
                        PredictionColumns::getColSort, PredictionColumns::getDisplayFlag)
                .eq(PredictionColumns::getPredictionId, predictionId));
        struTreeResp.setTableMap(tableMap);
        struTreeResp.setColumnsList(predictionColumnList.stream().map(column -> {
            PredictionColumnsResp predictionColumnsResp = new PredictionColumnsResp();
            predictionColumnsResp.setId(column.getId());
            predictionColumnsResp.setColName(column.getColName());
            predictionColumnsResp.setColSort(column.getColSort());
            predictionColumnsResp.setDisplayFlag(column.getDisplayFlag());
            predictionColumnsResp.setColType(column.getColType());
            return predictionColumnsResp;
        }).toList());
        List<PredictionStruTreeResp> list = predictionStruTreeList.stream().map(tree -> {
            List<PredictionTable> tables = tableMap.get(tree.getStruId());
            PredictionStruTreeResp resp = new PredictionStruTreeResp();
            resp.setId(tree.getId());
            resp.setStruId(tree.getStruId());
            resp.setName(tree.getName());
            resp.setRpId(tree.getPredictionId());
            resp.setLeftNo(tree.getLeftNo());
            resp.setRightNo(tree.getRightNo());
            resp.setParentStruId(tree.getParentStruId());
            resp.setStruTypeCode(tree.getStruTypeCode());
            resp.setOpen(true);
            resp.setTableId(!CollectionUtils.isEmpty(tables) ? tables.get(0).getId() : null);
            return resp;
        }).sorted(Comparator.comparing(PredictionStruTreeResp::getLeftNo)).toList();
        struTreeResp.setPredictionStruTreeList(list);
        return struTreeResp;
    }

    /**
     * @Description 新增结构
     * @param saveStructureReq
     * @return void
     * @Author nijun
     * @Date: 2024/9/2 15:05
     */
    @Override
    @UpdatePrediction
    @Transactional(rollbackFor = Exception.class)
    public PredictionInfo addStructure(SaveStructureReq saveStructureReq) {
        PredictionInfo predictionInfo = predictionInfoMapper.selectById(saveStructureReq.getRpId());
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        String struId = saveStructureReq.getStruId();
        String predictionId = saveStructureReq.getRpId();
        List<String[]> structures = saveStructureReq.getStructures();
        List<String> newStruList = Lists.newArrayList();
        PredictionStruTree predictionStruTree = predictionStruTreeMapper.selectOne(new LambdaQueryWrapper<PredictionStruTree>()
                .eq(PredictionStruTree::getPredictionId, predictionId)
                .eq(PredictionStruTree::getStruId, struId));
        if (ObjectUtils.isEmpty(predictionStruTree)) {
            throw new NationrelBootException("数据不存在");
        }

        PredictionStruTree parentStruNode = null;
        if (predictionStruTree.getParentStruId().equals(RpConstant.ROOTNODE_PARENT) || StringUtils.isEmpty(structures.get(0)[0])) {
            parentStruNode = predictionStruTree;
        } else {
            parentStruNode = predictionStruTreeMapper.selectOne(new LambdaQueryWrapper<PredictionStruTree>()
                    .eq(PredictionStruTree::getPredictionId, predictionId)
                    .eq(PredictionStruTree::getStruId, predictionStruTree.getParentStruId()));
        }
        List<PredictionStruTree> struChildrenTreeList = Lists.newArrayList();
        PredictionStruTree rootStrTree = new PredictionStruTree();
        rootStrTree.setPredictionId(predictionId);
        rootStrTree.setStruId(parentStruNode.getStruId());
        rootStrTree.setParentStruId(parentStruNode.getParentStruId());
        rootStrTree.setStruTypeCode(RpConstant.STRU_TYPE_STRU);
        rootStrTree.setName(parentStruNode.getName());
        struChildrenTreeList.add(rootStrTree);
        // 查询当前选中节点最后一个结构子节点，取它的右序号+1作为多级添加的起始左序号
        PredictionStruTree lastChildStruNode = predictionStruTreeMapper.selectOne(new LambdaQueryWrapper<PredictionStruTree>()
                .eq(PredictionStruTree::getPredictionId, predictionId)
                .eq(PredictionStruTree::getParentStruId, parentStruNode.getStruId())
                .orderByDesc(PredictionStruTree::getLeftNo)
                .last("limit 1"));
        Integer rightNo;
        if (lastChildStruNode != null) {
            rightNo = lastChildStruNode.getRightNo() + 1;
        } else {
            rightNo = parentStruNode.getRightNo();
        }

        for (String[] structure : structures) {
            String parentStruId = null;
            for (int i = 0; i < structure.length; i++) {
                String rootStruId = null;

                if (parentStruNode.getParentStruId().equals(RpConstant.ROOTNODE_PARENT)) {
                    rootStruId = parentStruNode.getStruId();
                } else {
                    rootStruId = parentStruNode.getParentStruId();
                }
                String childStruId = UuidUtils.getUUID();
                if (i == 0 && StringUtils.isNotEmpty(structure[i])) {
                    PredictionStruTree struTree = this.buildStruTree(predictionId, childStruId, rootStruId, structure[i]);
                    parentStruId = struTree.getId();
                    struChildrenTreeList.add(struTree);
                    newStruList.add(struTree.getStruId());
                } else {
                    if (StringUtils.isNotEmpty(structure[i])) {
                        if (StringUtils.isEmpty(parentStruId)) {
                            parentStruId = predictionStruTree.getStruId();
                        }
                        PredictionStruTree struTree = this.buildStruTree(predictionId, childStruId, parentStruId, structure[i]);
                        struChildrenTreeList.add(struTree);
                        newStruList.add(struTree.getStruId());
                        parentStruId = childStruId;
                    }
                }

            }
        }
        Map<String, PredictionStruTreeImport> struTreeMap = this.buildStrutsTree(struChildrenTreeList);
        // 获取根节点
        PredictionStruTreeImport rootNode = struTreeMap.get(parentStruNode.getStruId());
        if (ObjectUtils.isNotEmpty(lastChildStruNode)) {
            updateLeftAndRightNo(rootNode, lastChildStruNode.getRightNo());
        } else {
            updateLeftAndRightNo(rootNode, parentStruNode.getLeftNo());
        }
        updateDeep(rootNode, parentStruNode.getDeep());
        updateSeriesNo(rootNode, parentStruNode.getSeriesNo());
        updateType(rootNode);
        // 有需要插入的数据
        if (!CollectionUtils.isEmpty(newStruList)) {
            // 通过TreeMap构建treeList
            List<PredictionStruTree> struTreeList = new ArrayList<>();
            buildStruTreeList(rootNode, struTreeList, newStruList);

            int nodeCnt = newStruList.size() * 2;
            // 2.更新树节点-父级节点类型
            if (RpConstant.TREENODE_TYPE_LEAF.equals(parentStruNode.getType())) {
                parentStruNode.setType(RpConstant.TREENODE_TYPE_NONLEAF);
                parentStruNode.setUpdateUserBy(sysUser.getUsername());
                parentStruNode.setUpdateTime(new Date());
                predictionStruTreeMapper.updateById(parentStruNode);
            }

            // 计算偏移量
            // 3.更新树节点-左右序号维护
            predictionStruTreeMapper.updateLeftNoByChangeNum(rightNo, nodeCnt, predictionId);
            predictionStruTreeMapper.updateRightNoByChangeNum(rightNo, nodeCnt, predictionId);

            // 依次插入已读入的数据
            // 插入结构树数据
            this.saveStruTree(struTreeList, sysUser.getUsername());
        }
        return predictionInfo;
    }

    /**
     * @Description 删除结构
     * @param predictionId
     * @param struId
     * @return void
     * @Author nijun
     * @Date: 2024/9/2 15:05
     */
    @Override
    @UpdatePrediction
    @Transactional(rollbackFor = Exception.class)
    public PredictionInfo deleteStructure(String predictionId, String struId) {
        PredictionInfo predictionInfo = predictionInfoMapper.selectById(predictionId);
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        String uuid = UuidUtils.getUUID();
        // 取得当前节点
        PredictionStruTree predictionStruTree = predictionStruTreeMapper.selectOne(new LambdaQueryWrapper<PredictionStruTree>()
                .eq(PredictionStruTree::getPredictionId, predictionId)
                .eq(PredictionStruTree::getStruId, struId));
        if (ObjectUtils.isNotEmpty(predictionStruTree)) {
            PredictionStruTree tree = predictionStruTreeMapper.selectOne(new LambdaQueryWrapper<PredictionStruTree>()
                    .eq(PredictionStruTree::getPredictionId, predictionId)
                    .eq(PredictionStruTree::getStruId, struId));
            int leftNo = predictionStruTree.getLeftNo();
            int rightNo = predictionStruTree.getRightNo();
            int version = tree.getVersionNo();
            tree.setRecycleId(uuid);
            tree.setIsDelete(RpConstant.IS_DELETE);
            tree.setUpdateUserBy(sysUser.getUsername());
            tree.setUpdateTime(new Date());
            tree.setVersionNo(tree.getVersionNo() + 1);
            int update = predictionStruTreeMapper.update(tree, new LambdaUpdateWrapper<PredictionStruTree>()
                    .eq(PredictionStruTree::getPredictionId, predictionId)
                    .eq(PredictionStruTree::getVersionNo, version)
                    .eq(PredictionStruTree::getStruId, struId));
            List<PredictionStruTree> childStruTree = predictionStruTreeMapper.selectList(new LambdaUpdateWrapper<PredictionStruTree>()
                    .eq(PredictionStruTree::getPredictionId, predictionId)
                    .ge(PredictionStruTree::getLeftNo, leftNo)
                    .le(PredictionStruTree::getLeftNo, rightNo));
            childStruTree.forEach(child -> {
                int childVersion = child.getVersionNo();
                child.setRecycleId(uuid);
                child.setIsDelete(RpConstant.IS_DELETE);
                child.setUpdateUserBy(sysUser.getUsername());
                child.setUpdateTime(new Date());
                child.setVersionNo(child.getVersionNo() + 1);
                int childUpdate = predictionStruTreeMapper.update(child, new LambdaUpdateWrapper<PredictionStruTree>()
                        .eq(PredictionStruTree::getPredictionId, predictionId)
                        .eq(PredictionStruTree::getVersionNo, childVersion)
                        .eq(PredictionStruTree::getStruId, child.getStruId()));
                if (childUpdate == 0) {
                    throw new NationrelBootException("当前节点有变更，刷新后再操作");
                }
            });
            if (update == 0) {
                throw new NationrelBootException("当前节点有变更，刷新后再操作");
            }

            PredictionStruTree parentTreeNode = predictionStruTreeMapper.selectOne(new LambdaQueryWrapper<PredictionStruTree>()
                    .eq(PredictionStruTree::getPredictionId, predictionId)
                    .eq(PredictionStruTree::getStruId, tree.getParentStruId()));
            int changeNum = rightNo - leftNo + 1;
            if ((parentTreeNode.getRightNo() - parentTreeNode.getLeftNo() - 1) == changeNum) {
                parentTreeNode.setType(RpConstant.TREENODE_TYPE_LEAF);
                parentTreeNode.setUpdateUserBy(sysUser.getUsername());
                parentTreeNode.setUpdateTime(new Date());
                predictionStruTreeMapper.updateById(parentTreeNode);
            }
        }
        return predictionInfo;
    }

    /**
     * @Description 粘贴
     * @param pasteStructureReq
     * @return void
     * @Author nijun
     * @Date: 2024/9/2 15:05
     */
    @Override
    @UpdatePrediction
    @Transactional(rollbackFor = Exception.class)
    public PredictionInfo pasteStructure(PasteStructureReq pasteStructureReq) throws Exception {
        PredictionInfo predictionInfo = predictionInfoMapper.selectById(pasteStructureReq.getPredictionId());
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        PredictionStruTree copyStruTree = predictionStruTreeMapper.selectOne(new LambdaQueryWrapper<PredictionStruTree>()
                .eq(PredictionStruTree::getPredictionId, pasteStructureReq.getPredictionId())
                .eq(PredictionStruTree::getStruId, pasteStructureReq.getSrcStruId()));
        if (ObjectUtils.isEmpty(copyStruTree)) {
            throw new NationrelBootException("目标节点不存在");
        }
        PredictionStruTree selectStruTree = predictionStruTreeMapper.selectOne(new LambdaQueryWrapper<PredictionStruTree>()
                .eq(PredictionStruTree::getPredictionId, pasteStructureReq.getPredictionId())
                .eq(PredictionStruTree::getStruId, pasteStructureReq.getStruId()));
        if (ObjectUtils.isEmpty(selectStruTree)) {
            throw new NationrelBootException("选择节点不存在");
        }
        if (pasteStructureReq.getOptType() == 2) {
            // 判断被剪切的节点是否为父子关系（父节点不能剪切到子节点）
            Long count = predictionStruTreeMapper.selectCount(new LambdaQueryWrapper<PredictionStruTree>()
                    .eq(PredictionStruTree::getPredictionId, pasteStructureReq.getPredictionId())
                    .eq(PredictionStruTree::getStruId, selectStruTree.getStruId())
                    .gt(PredictionStruTree::getLeftNo, copyStruTree.getLeftNo())
                    .lt(PredictionStruTree::getLeftNo, copyStruTree.getRightNo()));
            if (count > 0) {
                throw new NationrelBootException("父节点不能剪切到子节点");
            }
        }

        // 查询指定结构下第一个结构的左序号
        PredictionStruTree firstStruTree = predictionStruTreeMapper.selectOne(new LambdaQueryWrapper<PredictionStruTree>()
                .eq(PredictionStruTree::getPredictionId, pasteStructureReq.getPredictionId())
                .ne(PredictionStruTree::getIsDelete, RpConstant.IS_DELETE)
                .eq(PredictionStruTree::getParentStruId, selectStruTree.getStruId())
                .orderByAsc(PredictionStruTree::getLeftNo)
                .last("limit 0, 1"));

        PredictionStruTree parentStruTree = predictionStruTreeMapper.selectOne(new LambdaQueryWrapper<PredictionStruTree>()
                .eq(PredictionStruTree::getPredictionId, pasteStructureReq.getPredictionId())
                .eq(PredictionStruTree::getStruId, pasteStructureReq.getStruId()));

        // 1、批量插入复制节点，各节点标识要重新生成，由于属于新建节点，原来关系如果保留，会产生网的冲突。因此，不拷贝关系；（原则，网关系在节点做删除，复制新建操作时，不保留（除合并））
        if (pasteStructureReq.getOptType() == 1) {
            int no = 0;
            if (ObjectUtils.isNotEmpty(firstStruTree)) {
                no = firstStruTree.getRightNo();
            } else {
                no = selectStruTree.getRightNo();
            }

            // 拷贝
            this.copy(sysUser, parentStruTree, pasteStructureReq.getPredictionId(),
                    pasteStructureReq.getStruId(), copyStruTree,
                    pasteStructureReq.getSrcStruId(), no);
        } else if (pasteStructureReq.getOptType() == 2) {
            this.cut(sysUser, parentStruTree, pasteStructureReq.getPredictionId(), parentStruTree.getStruId(), copyStruTree, copyStruTree.getStruId());
        }
        return predictionInfo;
    }

    /**
     * @Description 构建结构树
     * @param rpId
     * @param struId
     * @param pid
     * @param name
     * @return com.nationrel.modules.rp.entity.PredictionStruTree
     * @Author nijun
     * @Date: 2024/9/2 15:06
     */
    private PredictionStruTree buildStruTree(String rpId, String struId, String pid, String name) {
        PredictionStruTree struTree = new PredictionStruTree();
        struTree.setPredictionId(rpId);
        struTree.setStruId(struId);
        struTree.setParentStruId(pid);
        struTree.setName(name);
        struTree.setStruTypeCode(RpConstant.STRU_TYPE_STRU);
        return struTree;
    }

    private Map<String, PredictionStruTreeImport> buildStrutsTree(List<PredictionStruTree> treeList) {
        Map<String, PredictionStruTreeImport> importStruTreeMap = Maps.newTreeMap();
        // 将所有节点先放入map
        for (PredictionStruTree struTree : treeList) {
            String struId = struTree.getStruId();
            PredictionStruTreeImport importStruTree = new PredictionStruTreeImport();
            importStruTree.setPredictionStruTree(struTree);
            importStruTreeMap.put(struId, importStruTree);
        }
        // 根据上面放进去的节点，将所有节点分父子关系存入map
        for (PredictionStruTree struTree : treeList) {
            String struId = struTree.getStruId();
            String parentStruId = struTree.getParentStruId();
            PredictionStruTreeImport importStruTree = importStruTreeMap.get(parentStruId);
            if (importStruTree != null) {
                List<PredictionStruTreeImport> children = importStruTree.getChildren();
                if (CollectionUtils.isEmpty(children)) {
                    children = new ArrayList<>();
                    PredictionStruTreeImport importStruTree1 = importStruTreeMap.get(struId);
                    children.add(importStruTree1);
                } else {
                    PredictionStruTreeImport importStruTree1 = importStruTreeMap.get(struId);
                    children.add(importStruTree1);
                }
                importStruTree.setChildren(children);
            }
        }
        return importStruTreeMap;
    }

    /**
     * @Description 更新左右序号
     * @param importStruTree
     * @param leftNo
     * @return java.lang.Integer
     * @Author nijun
     * @Date: 2024/9/2 15:06
     */
    private Integer updateLeftAndRightNo(PredictionStruTreeImport importStruTree, Integer leftNo) {
        PredictionStruTree struTree = importStruTree.getPredictionStruTree();
        List<PredictionStruTreeImport> children = importStruTree.getChildren();

        Integer lastNo = leftNo;

        struTree.setLeftNo(leftNo);
        if (!CollectionUtils.isEmpty(children)) {
            // 结构
            for (PredictionStruTreeImport child : children) {
                String struTypeCode = child.getPredictionStruTree().getStruTypeCode();
                if (RpConstant.STRU_TYPE_STRU.equals(struTypeCode)) {
                    lastNo = updateLeftAndRightNo(child, lastNo + 1);
                }
            }
        }
        struTree.setRightNo(lastNo + 1);
        return struTree.getRightNo();
    }

    /**
     * @Description 更新结构层级
     * @param importStruTree
     * @param deep
     * @return void
     * @Author nijun
     * @Date: 2024/9/2 15:07
     */
    private void updateDeep(PredictionStruTreeImport importStruTree, Integer deep) {
        PredictionStruTree struTree = importStruTree.getPredictionStruTree();
        List<PredictionStruTreeImport> children = importStruTree.getChildren();
        struTree.setDeep(deep);
        if (!CollectionUtils.isEmpty(children)) {
            for (PredictionStruTreeImport child : children) {
                updateDeep(child, deep + 1);
            }
        }
    }

    /**
     * @Description 更新序列号
     * @param importStruTree
     * @param seriesNo
     * @return void
     * @Author nijun
     * @Date: 2024/9/2 15:07
     */
    private void updateSeriesNo(PredictionStruTreeImport importStruTree, String seriesNo) {
        PredictionStruTree struTree = importStruTree.getPredictionStruTree();
        List<PredictionStruTreeImport> children = importStruTree.getChildren();
        struTree.setSeriesNo(seriesNo);
        if (!CollectionUtils.isEmpty(children)) {
            // 结构
            int struIndex = 1;
            for (PredictionStruTreeImport child : children) {
                String struTypeCode = child.getPredictionStruTree().getStruTypeCode();
                if (RpConstant.STRU_TYPE_STRU.equals(struTypeCode)) {
                    String childSeriesNo;
                    if (RpConstant.ROOTNODE_PARENT.equals(child.getPredictionStruTree().getParentStruId())) {
                        childSeriesNo = "1";
                    } else {
                        childSeriesNo = seriesNo + "." + struIndex;
                    }
                    struIndex++;
                    updateSeriesNo(child, childSeriesNo);
                }
            }
        }
    }

    /**
     * @Description 更新节点是否为叶子节点
     * @param importStruTree
     * @return void
     * @Author nijun
     * @Date: 2024/9/2 15:07
     */
    private void updateType(PredictionStruTreeImport importStruTree) {
        PredictionStruTree struTree = importStruTree.getPredictionStruTree();
        List<PredictionStruTreeImport> children = importStruTree.getChildren();
        struTree.setType(RpConstant.TREENODE_TYPE_NONLEAF);
        if (CollectionUtils.isEmpty(children)) {
            struTree.setType(RpConstant.TREENODE_TYPE_LEAF);
            return;
        }
        for (PredictionStruTreeImport child : children) {
            updateType(child);
        }
    }

    /**
     * @Description 构建父子结构树
     * @param predictionStruTreeImport
     * @param struTreeList
     * @param newStruList
     * @return void
     * @Author nijun
     * @Date: 2024/9/2 15:07
     */
    private void buildStruTreeList(PredictionStruTreeImport predictionStruTreeImport, List<PredictionStruTree> struTreeList, List<String> newStruList) {
        List<PredictionStruTreeImport> children = predictionStruTreeImport.getChildren();
        if (newStruList.contains(predictionStruTreeImport.getPredictionStruTree().getStruId())) {
            struTreeList.add(predictionStruTreeImport.getPredictionStruTree());
        }
        if (!CollectionUtils.isEmpty(children)) {
            for (PredictionStruTreeImport child : children) {
                buildStruTreeList(child, struTreeList, newStruList);
            }
        }
    }

    /**
     * 插入结构树表
     * @param struTreeList
     */
    private void saveStruTree(List<PredictionStruTree> struTreeList, String userName) {
        List<PredictionStruTree> batchStruTrees = new ArrayList<>();
        for (PredictionStruTree struTree: struTreeList) {
            struTree.setInsertUserBy(userName);
            struTree.setInsertTime(new Date());
            struTree.setIsDelete(RpConstant.NOT_DELETE);
            batchStruTrees.add(struTree);
        }
        this.saveBatch(batchStruTrees);
    }

    /**
     * @Description 复制
     * @param user
     * @param parentStruTree
     * @param predictionId
     * @param parentStruId
     * @param copyStruTree
     * @param copyStruId
     * @param firstStruLeftNo
     * @return java.util.Map<java.lang.String,java.lang.String>
     * @Author nijun
     * @Date: 2024/9/2 15:08
     */
    private Map<String, String> copy(LoginUser user, PredictionStruTree parentStruTree, String predictionId,
                                     String parentStruId, PredictionStruTree copyStruTree, String copyStruId,
                                     Integer firstStruLeftNo) throws Exception {
        // 存放旧struid与newstruid信息
        Map<String, String> newStruIdMap = null;
        // 1.1 计算出要批量插入节点的数据，并在目标节点处预留，更改目标节点的类型
        // a、更改目标节点的类型
        if (parentStruTree.getRightNo() - parentStruTree.getLeftNo() == 1) {
            parentStruTree.setType(RpConstant.TREENODE_TYPE_NONLEAF);
            parentStruTree.setUpdateUserBy(user.getUsername());
            parentStruTree.setUpdateTime(new Date());
            predictionStruTreeMapper.updateById(parentStruTree);
        }
        int rightNo = parentStruTree.getRightNo();
        PredictionStruTree sonNodeOfStruTree = predictionStruTreeMapper.selectOne(new LambdaQueryWrapper<PredictionStruTree>()
                .eq(PredictionStruTree::getPredictionId, predictionId)
                .eq(PredictionStruTree::getParentStruId, parentStruId)
                .orderByDesc(PredictionStruTree::getLeftNo)
                .last("limit 1"));
        if (ObjectUtils.isNotEmpty(sonNodeOfStruTree)) {
            rightNo = sonNodeOfStruTree.getRightNo() + 1;
        }

        int nodeCnt = copyStruTree.getRightNo() - copyStruTree.getLeftNo() + 1;
        predictionStruTreeMapper.updateLeftNoByChangeNum(rightNo, nodeCnt, predictionId);
        predictionStruTreeMapper.updateRightNoByChangeNum(rightNo, nodeCnt, predictionId);
        copyStruTree = predictionStruTreeMapper.selectOne(new LambdaQueryWrapper<PredictionStruTree>()
                .eq(PredictionStruTree::getPredictionId, predictionId)
                .eq(PredictionStruTree::getStruId, copyStruId));
        newStruIdMap = this.copyAllChildTreeReturnNewStruId(parentStruTree, copyStruTree, firstStruLeftNo, user);
        // 将新生成对应拷贝节点的id取出来，留作刷新后选中节点的标志
        return newStruIdMap;
    }

    private Map<String, String> copyAllChildTreeReturnNewStruId(PredictionStruTree parentStruTree, PredictionStruTree copyStruTree,
                                                                int startLeftNo, LoginUser user) {
        Map<String, String> newStruIdMap = new HashMap<String, String>();
        // a.左右序，深度，序号的变化特征
        int offsetDeep = parentStruTree.getDeep() - copyStruTree.getDeep() + 1;

        // b.取得copyRdfmStruTree节点下所有子节点，并且进行左右序，深度，序号，和标识更换
        String copyPredictionId = copyStruTree.getPredictionId();
        int copyLeftNo = copyStruTree.getLeftNo();
        int copyRightNo = copyStruTree.getRightNo();
        String parentPredictionId = parentStruTree.getPredictionId();

        List<PredictionStruTree> importingTree = new ArrayList<>();
        // 导出范围的所有节点id集合
        Map<String, String> importStruIdMap = new HashMap<String, String>();
        List<PredictionStruTree> addTrees = predictionStruTreeMapper.selectList(new LambdaQueryWrapper<PredictionStruTree>()
                .eq(PredictionStruTree::getPredictionId, copyPredictionId)
                .eq(PredictionStruTree::getIsDelete, RpConstant.NOT_DELETE)
                .ge(PredictionStruTree::getLeftNo, copyLeftNo)
                .le(PredictionStruTree::getLeftNo, copyRightNo));

        // 父级节点
        Map<String, String> parentMapFlag = new HashMap<String, String>();
        for (PredictionStruTree tree : addTrees) {
            parentMapFlag.put(tree.getParentStruId(), tree.getStruId());
            // 将所有节点id保存到集合中，用于后面判断接口是否在导出范围
            importStruIdMap.put(tree.getStruId(), tree.getStruId());
        }
        // 进行左右序，深度，序号，和标识更换
        Stack<PredictionStruTree> stack = new Stack<PredictionStruTree>();
        int startLeft = startLeftNo;
        PredictionStruTree tmpNode = null;
        String oldId;
        String newId;
        for (PredictionStruTree tree : addTrees) {

            if (!stack.isEmpty()) {
                tmpNode = stack.peek();
                while (tmpNode.getRightNo() < tree.getLeftNo()) {
                    tmpNode = stack.pop();
                    tmpNode.setRightNo(startLeft++);
                    tmpNode = stack.peek();
                }

                // 判断是不是叶子结点
                if (parentMapFlag.containsKey(tree.getStruId())) {
                    // 不是叶子结点
                    tree.setLeftNo(startLeft++);
                    stack.push(tree);
                } else {
                    tree.setLeftNo(startLeft++);
                    tree.setRightNo(startLeft++);
                }

            } else {

                // 判断是不是叶子结点
                if (parentMapFlag.containsKey(tree.getStruId())) {
                    // 不是叶子结点
                    tree.setLeftNo(startLeft++);
                    stack.push(tree);
                } else {
                    tree.setLeftNo(startLeft++);
                    tree.setRightNo(startLeft++);
                }
            }

            // 原结构标识
            oldId = tree.getStruId();
            // 新结构标识
            newId = UuidUtils.getUUID();
            // 记录新旧标识对应关系
            newStruIdMap.put(oldId, newId);
            // 将新接口的起始节点和结束节点存入map中(创建新接口时使用)
            importStruIdMap.put(oldId, newId);
            // 将原struid替换为新的struid
            tree.setStruId(newId);
            tree.setPredictionId(parentPredictionId);
            tree.setDeep(tree.getDeep() + offsetDeep);
//			tree.setSeriesNo(reviseSeriesNo(tree.getSeriesNo(), copyRdfmStruTree.getSeriesNo(), seriesNo));
            importingTree.add(tree);
        }

        while (!stack.isEmpty()) {
            tmpNode = stack.pop();
            tmpNode.setRightNo(startLeft++);
        }

        // c.更新节点的上级节点标识
        String rootNodeParentStruId = copyStruTree.getParentStruId();
        String newRootNodeParentId = parentStruTree.getStruId();
        List<PredictionStruTree> newTree = new ArrayList<>();
        for (PredictionStruTree tree : importingTree) {
            oldId = tree.getParentStruId();
            tree.setId(null);
            // update5-导入结构树的结构标识和父级结构标识
            tree.setParentStruId(newStruIdMap.get(oldId));
            // up6-导入结构树根节点的父级结构标识
            if (rootNodeParentStruId.equals(oldId)) {
                tree.setParentStruId(newRootNodeParentId);
            }
            tree.setInsertUserBy(user.getUsername());
            tree.setInsertTime(new Date());
            newTree.add(tree);
        }
        this.saveBatch(newTree);
        importingTree.clear();
        return newStruIdMap;
    }

    /**
     * @Description 剪切
     * @param user
     * @param parentStruTree
     * @param predictionId
     * @param parentStruId
     * @param copyStruTree
     * @param copyStruId
     * @return void
     * @Author nijun
     * @Date: 2024/9/2 15:10
     */
    private void cut(LoginUser user, PredictionStruTree parentStruTree, String predictionId,
                     String parentStruId, PredictionStruTree copyStruTree, String copyStruId) throws Exception {
        PredictionStruTree rootStructNode = predictionStruTreeMapper.selectOne(new LambdaQueryWrapper<PredictionStruTree>()
                .eq(PredictionStruTree::getPredictionId, predictionId)
                .eq(PredictionStruTree::getParentStruId, RpConstant.ROOTNODE_PARENT));

        // (2.1)将拷贝节点排除在树外（排除方法左右序号都+根结点右序号+前次假删的节点数*2）。便于以后更新。
        predictionStruTreeMapper.updateLeftRightNoBecauseRemove(predictionId, copyStruTree.getLeftNo(),
                copyStruTree.getRightNo(), rootStructNode.getRightNo(), 0);
        int offset = copyStruTree.getRightNo() - copyStruTree.getLeftNo() + 1;

        // (2.2)将拷贝节点排除在树外，余下节点左右序号重新顺处理；
        predictionStruTreeMapper.updateLeftNoInPartTree(predictionId, copyStruTree.getLeftNo(), rootStructNode.getRightNo(), -offset);
        predictionStruTreeMapper.updateRightNoInPartTree(predictionId, copyStruTree.getRightNo(), rootStructNode.getRightNo(), -offset);

        // (2.3)将拷贝节点排除在树外，原上级节点类型更改
        PredictionStruTree copyParentStruTree = predictionStruTreeMapper.selectOne(new LambdaQueryWrapper<PredictionStruTree>()
                .eq(PredictionStruTree::getPredictionId, predictionId)
                .eq(PredictionStruTree::getStruId, copyStruTree.getParentStruId()));
        if (copyParentStruTree.getRightNo() - copyParentStruTree.getLeftNo() == 1) {
            copyParentStruTree.setType(RpConstant.TREENODE_TYPE_LEAF);
            copyParentStruTree.setUpdateUserBy(user.getUsername());
            copyParentStruTree.setUpdateTime(new Date());
            predictionStruTreeMapper.updateById(copyParentStruTree);
        }

        // (2.5) 被粘贴节点的类型更改
        parentStruTree = predictionStruTreeMapper.selectOne(new LambdaQueryWrapper<PredictionStruTree>()
                .eq(PredictionStruTree::getPredictionId, predictionId)
                .eq(PredictionStruTree::getStruId, parentStruId));
        if (RpConstant.TREENODE_TYPE_LEAF.equals(parentStruTree.getType())) {
            parentStruTree.setType(RpConstant.TREENODE_TYPE_NONLEAF);
            parentStruTree.setUpdateUserBy(user.getUsername());
            parentStruTree.setUpdateTime(new Date());
            predictionStruTreeMapper.updateById(parentStruTree);
        }

        // 取得剪切节点在新上级节点下的位置
        int rightNo = parentStruTree.getRightNo();
        PredictionStruTree sonNodeOfStruTree = predictionStruTreeMapper.selectOne(new LambdaQueryWrapper<PredictionStruTree>()
                .eq(PredictionStruTree::getPredictionId, predictionId)
                .eq(PredictionStruTree::getParentStruId, parentStruId)
                .gt(PredictionStruTree::getLeftNo, parentStruTree.getLeftNo())
                .lt(PredictionStruTree::getRightNo, parentStruTree.getRightNo())
                .orderByDesc(PredictionStruTree::getLeftNo)
                .last("limit 1"));
        if (ObjectUtils.isNotEmpty(sonNodeOfStruTree)) {
            // 兄弟节点的最大右序号
            rightNo = sonNodeOfStruTree.getRightNo() + 1;
        }

        // (2.6) 被粘贴节点下， 左右序号，被剪切节点数量的预留
        predictionStruTreeMapper.updateLeftNoInPartTree(predictionId, rightNo, rootStructNode.getRightNo(), offset);
        predictionStruTreeMapper.updateRightNoInPartTree(predictionId, rightNo, rootStructNode.getRightNo(), offset);

        copyStruTree = predictionStruTreeMapper.selectOne(new LambdaQueryWrapper<PredictionStruTree>()
                .eq(PredictionStruTree::getPredictionId, predictionId)
                .eq(PredictionStruTree::getStruId, copyStruId));
        int minusOffset = copyStruTree.getLeftNo() - (rightNo);
        // (2.7) 剪切节点的上级节点修改, 左右序号，深度，序号
        predictionStruTreeMapper.updateParentStructureId(predictionId, copyStruTree.getLeftNo(), copyStruTree.getRightNo(),
                copyStruTree.getParentStruId(), parentStruId);

        predictionStruTreeMapper.updateLeftRightNoBecauseRemove(predictionId, copyStruTree.getLeftNo(),
                copyStruTree.getRightNo(), -minusOffset, parentStruTree.getDeep() - copyStruTree.getDeep() + 1);
    }
}
